package fr.doap.llog;

import java.text.DateFormat;
import java.util.Date;

public interface ILogger {
	public void logHead(LogMsg aLogMsg, StackTraceElement aSTE);
	public void logCauseHead(LogMsg aLogMsg, Throwable aThrowable);
	public void logSTLine(LogMsg aLogMsg, StackTraceElement aSTE);

	final static public ILogger	CONSOLE = new Console();
	
	static public class Console implements ILogger {
		@Override
		public void logHead(LogMsg aLogMsg, StackTraceElement aSTE) {
			String lC = aSTE.getClassName()+" @"+aSTE.getMethodName();
			String lP = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.MEDIUM).format(new Date(aLogMsg.mTime))+" "+aLogMsg.mCallerThread.getName()+" "+L.getSeverity(aLogMsg.mSeverity).substring(0,1)+"/"+aLogMsg.mTag;
			if (aLogMsg.mSeverity >= L.ERROR) System.err.println(lP+ " - " + lC + ": " + aLogMsg.mMessage + " (" + aSTE.getFileName() + ":" + aSTE.getLineNumber() + ")");
			else System.out.println(lP+ " - " + lC + ": " + aLogMsg.mMessage + " (" + aSTE.getFileName() + ":" + aSTE.getLineNumber() + ")");
		}
		
		@Override
		public void logCauseHead(LogMsg aLogMsg, Throwable aThrowable) {
			StackTraceElement lSTE = aThrowable.getStackTrace()[0];
			String lC = lSTE.getClassName() + "." + lSTE.getMethodName();
			String lMsg = aThrowable.getLocalizedMessage();
			if (aLogMsg.mSeverity >= L.ERROR) System.err.println("caused by " + lC + ": " + lMsg + " (" + lSTE.getFileName() + ":" + lSTE.getLineNumber() + ")");
			else System.out.println("caused by " + lC + ": " + lMsg + " (" + lSTE.getFileName() + ":" + lSTE.getLineNumber() + ")");
		}
		
		@Override
		public void logSTLine(LogMsg aLogMsg, StackTraceElement aSTE) {
			String lC = aSTE.getClassName()+"."+aSTE.getMethodName();
			if (aLogMsg.mSeverity >= L.ERROR) System.err.println("  called by "+lC + " (" + aSTE.getFileName() + ":" + aSTE.getLineNumber() + ")");
			else System.out.println("  called by "+lC + " (" + aSTE.getFileName() + ":" + aSTE.getLineNumber() + ")");
		}
	}
}