package fr.doap.llog;

import java.io.*;
import java.util.*;

public class LogLooper extends ThreadLooper {

	public LogLooper() {
		reloadConfig();
	}
	
	public void post(LogMsg aLogMsg) {
		add(new LLMsg(aLogMsg));
	}
	
	public void put(String aKey, Filter aF) {
		mFilters.put(aKey, aF);
	}
	
	public void setLogger(ILogger aLogger) {
		mLogger = aLogger;
	}
	
	public class LLMsg implements WorkItem {
		public LLMsg(LogMsg aLogMsg) { mLogMsg = aLogMsg; }
		
		@Override
		public void run() {
			int lI = mLogMsg.getCallerIndex();
			StackTraceElement lS = mLogMsg.mSTE[lI];
			String lClass = lS.getClassName();
			lClass = lClass.substring(lClass.lastIndexOf('.')+1);
			if (mLogMsg.mTag == null || mLogMsg.mTag.isEmpty()) mLogMsg.mTag = lClass;

			String lC = lS.getClassName() + "." + lS.getMethodName();
			Filter lF = mFilters.get(lC);
			if (lF == null) lF = Filter.DEFAULT;
			boolean lShowST = false;
			if (mLogMsg.mSeverity >= lF.mLogLevel) {
				mLogger.logHead(mLogMsg, lS);
				if (mLogMsg.mSeverity >= lF.mSTLevel) {
					lShowST = true;
					for (int i=lI+1; i<mLogMsg.mSTE.length; i++) mLogger.logSTLine(mLogMsg, mLogMsg.mSTE[i]);
				}
			}
			if (mLogMsg.mThrowable != null) printLog(mLogMsg.mThrowable, lF, lShowST);
		}
		
		protected void printLog(Throwable aThrowable, Filter aFilter, boolean aShowST) {
			StackTraceElement[] lSTEs = aThrowable.getStackTrace();
			StackTraceElement lSTE = lSTEs[0];
			String lClass = lSTE.getClassName();
			lClass = lClass.substring(lClass.lastIndexOf('.'));
			String lC = lSTE.getClassName() + "." + lSTE.getMethodName();
			String lMsg = aThrowable.getLocalizedMessage();
			if (aFilter != null) {
				mLogger.logCauseHead(mLogMsg, aThrowable);
				if (aShowST) for (int i=1; i<lSTEs.length; i++) mLogger.logSTLine(mLogMsg, lSTEs[i]);
			} else {
				System.out.println("caused by " + lC + ": " + lMsg + " (" + lSTE.getFileName() + ":" + lSTE.getLineNumber() + ")");
			}
			if (aThrowable.getCause() != null) printLog(aThrowable.getCause(), aFilter, aShowST);
		}
		
		private LogMsg	mLogMsg;
	}
	
	public void reloadConfig() {
		Properties lProp = new Properties();
		File lF = new File("llog.properties");
		if (lF.exists()) {
			try {
				InputStream is = new FileInputStream(lF);
				lProp.load(is);
				for (String k : lProp.stringPropertyNames()) {
					String v = lProp.getProperty(k);
					String lv[] = v.split(",");
					int lLevel = L.getSeverity(lv[0].trim().toUpperCase());
					int lST = L.ERROR;
					if (lv.length > 1) lST = L.getSeverity(lv[1].trim().toUpperCase());
					put(k.trim(), new Filter(lLevel, lST));
				}
			} catch (Exception e) {
				System.out.println("Problem in file " + lF.getAbsolutePath()+": "+e.getMessage());
			}
		} else System.out.println("Can't load file " + lF.getAbsolutePath());
	}
	
	private	Dict<Filter>	mFilters = new Dict<>();
	private ILogger			mLogger = ILogger.CONSOLE;
}
