package fr.doap.llog;

import java.util.*;
import java.util.concurrent.ConcurrentLinkedQueue;

public class ThreadLooper {

	public interface WorkItem {
		void run();
	}
	
	public void add(WorkItem aWorkItem) {
		if (mLooperThread == null) createThreadLooper();
		
		Thread thread = Thread.currentThread();
		if (!mCallerThreads.contains(thread)&&(!thread.isDaemon())) addCallerThread(thread);
		
		mMessageQueue.add(aWorkItem);
		if (!mFlushing) {
			synchronized (this) {
				notify();
			}
		}
	}
	
	protected void onThreadInit() {}
	
	protected void onBeforeFlush() {}
	
	protected void onStartWaiting() {}
	
	protected void onShutdownThread() {}
	
	protected void onUncaughtException(Thread t, Throwable e) {
		/*
		L.e("Uncaught Exception '"+e+"' in thread '"+t.getName()+"' : flushing...");
		L.e("Uncaught Exception '"+e.getMessage()+"' in thread '"+t.getName()+"' : flushing...");
		L.e("Uncaught Exception '"+e.getLocalizedMessage()+"' in thread '"+t.getName()+"' : flushing...");
		 */
		System.out.println("Caller threads : "+ mCallerThreads);
		System.out.println("Removing "+t);
		mCallerThreads.remove(t);
		mLooperThread.interrupt();
		if (e.getCause() != null) System.out.println("Cause: "+e.getCause());
		e.printStackTrace();
	}
	
	protected synchronized void createThreadLooper() {
		mLooperThread = new Thread(new Runnable(){
			@Override
			public void run() {
				onThreadInit();
				
				while (!Thread.currentThread().isInterrupted()) {
					flushAll();
					synchronized (ThreadLooper.this) {
						try {
							mFlushing = false;
							onStartWaiting();
							ThreadLooper.this.wait();
						} catch(InterruptedException e) {
//							System.out.println("Interrupted");
//							System.out.println("Caller threads: "+ mCallerThreads);
							if (mCallerThreads.isEmpty()) Thread.currentThread().interrupt();
						}
					}
				}
				flushAll();
				onShutdownThread();
				mLooperThread = null;
			}
		});
		mLooperThread.setName(getClass().getSimpleName()+"-"+hashCode());
		mLooperThread.start();
	}
	
	private void flushAll() {
		if (!mFlushing) {
			mFlushing = true;
			onBeforeFlush();
			while (!mMessageQueue.isEmpty()) {
				WorkItem lWorkItem = mMessageQueue.poll();
				lWorkItem.run();
			}
		}
		mFlushing = false;
	}
	
	private void addCallerThread(Thread t) {
		mCallerThreads.add(t);
		new CustomUncaughtExceptionHandler(t);
		Thread join = new Thread() {
			@Override
			public void run() {
				try {
					t.join();
				} catch (InterruptedException e) {}
				mCallerThreads.remove(t);
				if ((mLooperThread != null)&&!mLooperThread.isInterrupted()) mLooperThread.interrupt();
			}
			
		};
		join.setName("Finalizer-"+getClass().getSimpleName()+"-"+hashCode());
		join.start();
	}
	
	private Thread 							mLooperThread;
	private ConcurrentLinkedQueue<WorkItem>	mMessageQueue = new ConcurrentLinkedQueue<>();
	private Set<Thread> 					mCallerThreads = new HashSet<>();
	private boolean							mFlushing = false;
	
	
	/**
	 * Classe pour les UncaughtExceptionHandler
	 */
	private class CustomUncaughtExceptionHandler implements Thread.UncaughtExceptionHandler {
		
		public CustomUncaughtExceptionHandler(Thread t) {
			mUncaughtExceptionHandler = t.getUncaughtExceptionHandler();
			t.setUncaughtExceptionHandler(this);
		}
		
		@Override
		public void uncaughtException(Thread t, Throwable e) {
			onUncaughtException(t, e);
			flushAll();
			mCallerThreads.remove(t);
			if ((mLooperThread != null)&&!mLooperThread.isInterrupted()) mLooperThread.interrupt();
			if (mUncaughtExceptionHandler != null) mUncaughtExceptionHandler.uncaughtException(t, e);
		}
		
		private Thread.UncaughtExceptionHandler mUncaughtExceptionHandler;
	}
}